@@ -43,6 +43,8 @@
 #include <gst/pbutils/pbutils.h>
 
 #include "gstplay-marshal.h"
+#include "gstplay-enum.h"
+#include "gstfactorylists.h"
 
 /* generic templates */
 static GstStaticPadTemplate decoder_bin_sink_template =
@@ -97,7 +99,7 @@ struct _GstDecodeBin
                                  * Should be freed in dispose */
   gint nbpads;                  /* unique identifier for source pads */
 
-  GList *factories;             /* factories we can use for selecting elements */
+  GValueArray *factories;       /* factories we can use for selecting elements */
 
   gboolean have_type;           /* if we received the have_type signal */
   guint have_type_id;           /* signal id for have-type from typefind */
@@ -120,9 +122,15 @@ struct _GstDecodeBinClass
   /* signal fired to get a list of factories to try to autoplug */
   GValueArray *(*autoplug_factories) (GstElement * element, GstPad * pad,
       GstCaps * caps);
+  /* signal fired to sort the factories */
+  GValueArray *(*autoplug_sort) (GstElement * element, GstPad * pad,
+      GstCaps * caps, GValueArray * factories);
   /* signal fired to select from the proposed list of factories */
-    gint (*autoplug_select) (GstElement * element, GstPad * pad, GstCaps * caps,
-      GValueArray * factories);
+    GstAutoplugSelectResult (*autoplug_select) (GstElement * element,
+      GstPad * pad, GstCaps * caps, GstElementFactory * factory);
+
+  /* fired when the last group is drained */
+  void (*drained) (GstElement * element);
 };
 
 /* signals */
@@ -134,6 +142,8 @@ enum
   SIGNAL_AUTOPLUG_CONTINUE,
   SIGNAL_AUTOPLUG_FACTORIES,
   SIGNAL_AUTOPLUG_SELECT,
+  SIGNAL_AUTOPLUG_SORT,
+  SIGNAL_DRAINED,
   LAST_SIGNAL
 };
 
@@ -165,8 +175,10 @@ static gboolean gst_decode_bin_autoplug_continue (GstElement * element,
     GstPad * pad, GstCaps * caps);
 static GValueArray *gst_decode_bin_autoplug_factories (GstElement *
     element, GstPad * pad, GstCaps * caps);
-static gint gst_decode_bin_autoplug_select (GstElement * element,
-    GstPad * pad, GstCaps * caps, GValueArray * list);
+static GValueArray *gst_decode_bin_autoplug_sort (GstElement * element,
+    GstPad * pad, GstCaps * caps, GValueArray * factories);
+static GstAutoplugSelectResult gst_decode_bin_autoplug_select (GstElement *
+    element, GstPad * pad, GstCaps * caps, GstElementFactory * factory);
 
 static void gst_decode_bin_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -205,7 +217,6 @@ static GstStateChangeReturn gst_decode_bin_change_state (GstElement * element,
  * Streams belonging to the same group/chain of a media file
  *
  */
-
 struct _GstDecodeGroup
 {
   GstDecodeBin *dbin;
@@ -222,8 +233,7 @@ struct _GstDecodeGroup
 
   GList *endpads;               /* List of GstDecodePad of source pads to be exposed */
   GList *ghosts;                /* List of GstGhostPad for the endpads */
-
-  GList *reqpads;               /* List of RequestPads */
+  GList *reqpads;               /* List of RequestPads for multiqueue. */
 };
 
 #define GROUP_MUTEX_LOCK(group) G_STMT_START {				\
@@ -333,6 +343,33 @@ _gst_boolean_accumulator (GSignalInvocationHint * ihint,
   return myboolean;
 }
 
+/* we collect the first result */
+static gboolean
+_gst_array_accumulator (GSignalInvocationHint * ihint,
+    GValue * return_accu, const GValue * handler_return, gpointer dummy)
+{
+  gpointer array;
+
+  array = g_value_get_boxed (handler_return);
+  if (!(ihint->run_type & G_SIGNAL_RUN_CLEANUP))
+    g_value_set_boxed (return_accu, array);
+
+  return FALSE;
+}
+
+static gboolean
+_gst_select_accumulator (GSignalInvocationHint * ihint,
+    GValue * return_accu, const GValue * handler_return, gpointer dummy)
+{
+  GstAutoplugSelectResult res;
+
+  res = g_value_get_enum (handler_return);
+  if (!(ihint->run_type & G_SIGNAL_RUN_CLEANUP))
+    g_value_set_enum (return_accu, res);
+
+  return FALSE;
+}
+
 static void
 gst_decode_bin_class_init (GstDecodeBinClass * klass)
 {
@@ -417,7 +454,13 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
    * @caps: The #GstCaps found.
    *
    * This function is emited when an array of possible factories for @caps on
-   * @pad is needed. Decodebin2 will by default return 
+   * @pad is needed. Decodebin2 will by default return an array with all
+   * compatible factories, sorted by rank. 
+   *
+   * If this function returns NULL, @pad will be exposed as a final caps.
+   *
+   * If this function returns an empty array, the pad will be considered as 
+   * having an unhandled type media type.
    *
    * Returns: a #GValueArray* with a list of factories to try. The factories are
    * by default tried in the returned order or based on the index returned by
@@ -426,10 +469,31 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
   gst_decode_bin_signals[SIGNAL_AUTOPLUG_FACTORIES] =
       g_signal_new ("autoplug-factories", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass,
-          autoplug_factories), NULL, NULL,
+          autoplug_factories), _gst_array_accumulator, NULL,
       gst_play_marshal_BOXED__OBJECT_OBJECT, G_TYPE_VALUE_ARRAY, 2,
       GST_TYPE_PAD, GST_TYPE_CAPS);
 
+  /**
+   * GstDecodeBin2::autoplug-sort:
+   * @pad: The #GstPad.
+   * @caps: The #GstCaps.
+   * @factories: A #GValueArray of possible #GstElementFactory to use.
+   *
+   * Once decodebin2 has found the possible #GstElementFactory objects to try
+   * for @caps on @pad, this signal is emited. The purpose of the signal is for
+   * the application to perform additional sorting or filtering on the element
+   * factory array.
+   *
+   * The callee should copy and modify @factories.
+   *
+   * Returns: A new sorted array of #GstElementFactory objects.
+   */
+  gst_decode_bin_signals[SIGNAL_AUTOPLUG_SORT] =
+      g_signal_new ("autoplug-sort", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass, autoplug_sort),
+      NULL, NULL, gst_play_marshal_BOXED__OBJECT_OBJECT_BOXED,
+      G_TYPE_VALUE_ARRAY, 3, GST_TYPE_PAD, GST_TYPE_CAPS, G_TYPE_VALUE_ARRAY);
+
   /**
    * GstDecodeBin2::autoplug-select:
    * @pad: The #GstPad.
@@ -442,14 +506,28 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
    *
    * Returns: A #gint indicating what factory index from the @factories array
    * that you wish decodebin2 to use for trying to decode the given @caps.
-   * -1 to stop selection of a factory. The default handler always
-   * returns the first possible factory.
+   * Return -1 to stop selection of a factory and expose the pad as a raw type. 
+   * The default handler always returns the first possible factory (index 0).
    */
   gst_decode_bin_signals[SIGNAL_AUTOPLUG_SELECT] =
       g_signal_new ("autoplug-select", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass, autoplug_select),
-      NULL, NULL, gst_play_marshal_INT__OBJECT_OBJECT_BOXED,
-      G_TYPE_INT, 3, GST_TYPE_PAD, GST_TYPE_CAPS, G_TYPE_VALUE_ARRAY);
+      _gst_select_accumulator, NULL,
+      gst_play_marshal_ENUM__OBJECT_OBJECT_OBJECT,
+      GST_TYPE_AUTOPLUG_SELECT_RESULT, 3, GST_TYPE_PAD, GST_TYPE_CAPS,
+      GST_TYPE_ELEMENT_FACTORY);
+
+  /**
+   * GstDecodeBin2::drained
+   *
+   * This signal is emitted once decodebin2 has finished decoding all the data.
+   *
+   * Since: 0.10.16
+   */
+  gst_decode_bin_signals[SIGNAL_DRAINED] =
+      g_signal_new ("drained", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass, drained),
+      NULL, NULL, gst_marshal_VOID__VOID, G_TYPE_NONE, 0, G_TYPE_NONE);
 
   g_object_class_install_property (gobject_klass, PROP_CAPS,
       g_param_spec_boxed ("caps", "Caps", "The caps on which to stop decoding.",
@@ -466,6 +544,7 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
       GST_DEBUG_FUNCPTR (gst_decode_bin_autoplug_continue);
   klass->autoplug_factories =
       GST_DEBUG_FUNCPTR (gst_decode_bin_autoplug_factories);
+  klass->autoplug_sort = GST_DEBUG_FUNCPTR (gst_decode_bin_autoplug_sort);
   klass->autoplug_select = GST_DEBUG_FUNCPTR (gst_decode_bin_autoplug_select);
 
   gst_element_class_add_pad_template (gstelement_klass,
@@ -479,80 +558,12 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
       GST_DEBUG_FUNCPTR (gst_decode_bin_change_state);
 }
 
-/* the filter function for selecting the elements we can use in
- * autoplugging */
-static gboolean
-gst_decode_bin_factory_filter (GstPluginFeature * feature,
-    GstDecodeBin * decode_bin)
-{
-  guint rank;
-  const gchar *klass;
-
-  /* we only care about element factories */
-  if (!GST_IS_ELEMENT_FACTORY (feature))
-    return FALSE;
-
-  klass = gst_element_factory_get_klass (GST_ELEMENT_FACTORY (feature));
-  /* only demuxers, decoders, depayloaders and parsers can play */
-  if (strstr (klass, "Demux") == NULL &&
-      strstr (klass, "Decoder") == NULL &&
-      strstr (klass, "Depayloader") == NULL &&
-      strstr (klass, "Parse") == NULL) {
-    return FALSE;
-  }
-
-  /* only select elements with autoplugging rank */
-  rank = gst_plugin_feature_get_rank (feature);
-  if (rank < GST_RANK_MARGINAL)
-    return FALSE;
-
-  return TRUE;
-}
-
-/* function used to sort element features */
-static gint
-compare_ranks (GstPluginFeature * f1, GstPluginFeature * f2)
-{
-  gint diff;
-  const gchar *rname1, *rname2;
-
-  diff = gst_plugin_feature_get_rank (f2) - gst_plugin_feature_get_rank (f1);
-  if (diff != 0)
-    return diff;
-
-  rname1 = gst_plugin_feature_get_name (f1);
-  rname2 = gst_plugin_feature_get_name (f2);
-
-  diff = strcmp (rname2, rname1);
-
-  return diff;
-}
-
-static void
-print_feature (GstPluginFeature * feature)
-{
-  const gchar *rname;
-
-  rname = gst_plugin_feature_get_name (feature);
-
-  GST_DEBUG ("%s", rname);
-}
-
 static void
 gst_decode_bin_init (GstDecodeBin * decode_bin)
 {
-  GList *factories;
-
   /* first filter out the interesting element factories */
-  factories = gst_default_registry_feature_filter (
-      (GstPluginFeatureFilter) gst_decode_bin_factory_filter,
-      FALSE, decode_bin);
-
-  /* sort them according to their ranks */
-  decode_bin->factories = g_list_sort (factories, (GCompareFunc) compare_ranks);
-  /* do some debugging */
-  g_list_foreach (decode_bin->factories, (GFunc) print_feature, NULL);
-
+  decode_bin->factories =
+      gst_factory_list_get_elements (GST_FACTORY_LIST_DECODER);
 
   /* we create the typefind element only once */
   decode_bin->typefind = gst_element_factory_make ("typefind", "typefind");
@@ -608,7 +619,7 @@ gst_decode_bin_dispose (GObject * object)
   decode_bin = GST_DECODE_BIN (object);
 
   if (decode_bin->factories)
-    gst_plugin_feature_list_free (decode_bin->factories);
+    g_value_array_free (decode_bin->factories);
   decode_bin->factories = NULL;
 
   if (decode_bin->activegroup) {
@@ -782,6 +793,8 @@ static gboolean
 gst_decode_bin_autoplug_continue (GstElement * element, GstPad * pad,
     GstCaps * caps)
 {
+  GST_DEBUG_OBJECT (element, "autoplug-continue returns TRUE");
+
   /* by default we always continue */
   return TRUE;
 }
@@ -795,18 +808,33 @@ gst_decode_bin_autoplug_factories (GstElement * element, GstPad * pad,
   /* return all compatible factories for caps */
   result = find_compatibles (GST_DECODE_BIN (element), pad, caps);
 
+  GST_DEBUG_OBJECT (element, "autoplug-factories returns %p", result);
+
   return result;
 }
 
-static gint
-gst_decode_bin_autoplug_select (GstElement * element, GstPad * pad,
+static GValueArray *
+gst_decode_bin_autoplug_sort (GstElement * element, GstPad * pad,
     GstCaps * caps, GValueArray * factories)
 {
-  g_return_val_if_fail (factories != NULL, -1);
-  g_return_val_if_fail (factories->n_values > 0, -1);
+  GValueArray *result;
 
-  /* Return first factory. */
-  return 0;
+  result = g_value_array_copy (factories);
+
+  GST_DEBUG_OBJECT (element, "autoplug-sort returns %p", result);
+
+  /* return input */
+  return result;
+}
+
+static GstAutoplugSelectResult
+gst_decode_bin_autoplug_select (GstElement * element, GstPad * pad,
+    GstCaps * caps, GstElementFactory * factory)
+{
+  GST_DEBUG_OBJECT (element, "default autoplug-select returns TRY");
+
+  /* Try factory. */
+  return GST_AUTOPLUG_SELECT_TRY;
 }
 
 /********
@@ -855,7 +883,7 @@ analyze_new_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
     GstCaps * caps, GstDecodeGroup * group)
 {
   gboolean apcontinue = TRUE;
-  GValueArray *factories = NULL;
+  GValueArray *factories = NULL, *result = NULL;
 
   GST_DEBUG_OBJECT (dbin, "Pad %s:%s caps:%" GST_PTR_FORMAT,
       GST_DEBUG_PAD_NAME (pad), caps);
@@ -886,12 +914,26 @@ analyze_new_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
   g_signal_emit (G_OBJECT (dbin),
       gst_decode_bin_signals[SIGNAL_AUTOPLUG_FACTORIES], 0, pad, caps,
       &factories);
-  if (factories == NULL) {
+
+  /* NULL means that we can expose the pad */
+  if (factories == NULL)
+    goto expose_pad;
+
+  /* if the array is empty, we have an unknown type */
+  if (factories->n_values == 0) {
     /* no compatible factories */
+    g_value_array_free (factories);
     goto unknown_type;
   }
 
-  /* 1.d else continue autoplugging something from the list. */
+  /* 1.d sort some more. */
+  g_signal_emit (G_OBJECT (dbin),
+      gst_decode_bin_signals[SIGNAL_AUTOPLUG_SORT], 0, pad, caps, factories,
+      &result);
+  g_value_array_free (factories);
+  factories = result;
+
+  /* 1.e else continue autoplugging something from the list. */
   GST_LOG_OBJECT (pad, "Let's continue discovery on this pad");
   connect_pad (dbin, src, pad, caps, factories, group);
 
@@ -994,37 +1036,45 @@ connect_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
 
     if (!(mqpad = gst_decode_group_control_demuxer_pad (group, pad)))
       goto beach;
+    src = group->multiqueue;
     pad = mqpad;
   }
 
   /* 2. Try to create an element and link to it */
   while (factories->n_values > 0) {
-    gint factidx;
+    GstAutoplugSelectResult ret;
     GstElementFactory *factory;
     GstElement *element;
     GstPad *sinkpad;
 
-    /* emit autoplug-select */
+    /* take first factory */
+    factory = g_value_get_object (g_value_array_get_nth (factories, 0));
+    /* Remove selected factory from the list. */
+    g_value_array_remove (factories, 0);
+
+    /* emit autoplug-select to see what we should do with it. */
     g_signal_emit (G_OBJECT (dbin),
         gst_decode_bin_signals[SIGNAL_AUTOPLUG_SELECT],
-        0, pad, caps, factories, &factidx);
+        0, pad, caps, factory, &ret);
 
-    /* when the function returns -1, we leave the loop and stop */
-    if (factidx == -1) {
-      GST_DEBUG_OBJECT (dbin, "autoplug select returned -1");
-      break;
+    switch (ret) {
+      case GST_AUTOPLUG_SELECT_TRY:
+        GST_DEBUG_OBJECT (dbin, "autoplug select requested try");
+        break;
+      case GST_AUTOPLUG_SELECT_EXPOSE:
+        GST_DEBUG_OBJECT (dbin, "autoplug select requested expose");
+        /* expose the pad, we don't have the source element */
+        expose_pad (dbin, src, pad, group);
+        res = TRUE;
+        goto beach;
+      case GST_AUTOPLUG_SELECT_SKIP:
+        GST_DEBUG_OBJECT (dbin, "autoplug select requested skip");
+        continue;
+      default:
+        GST_WARNING_OBJECT (dbin, "autoplug select returned unhandled %d", ret);
+        break;
     }
 
-    GST_DEBUG_OBJECT (dbin, "autoplug select returned %d", factidx);
-
-    factory = g_value_get_object (g_value_array_get_nth (factories, factidx));
-
-    /* factory is != NULL now since the default handler selects the first
-     * factory if all connected user handlers returned NULL. */
-
-    /* Remove selected factory from the list. */
-    g_value_array_remove (factories, factidx);
-
     /* 2.1. Try to create an element */
     if ((element = gst_element_factory_create (factory, NULL)) == NULL) {
       GST_WARNING_OBJECT (dbin, "Could not create an element from %s",
@@ -1032,7 +1082,18 @@ connect_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
       continue;
     }
 
-    /* 2.3. Find its sink pad */
+    /* ... activate it ... We do this before adding it to the bin so that we
+     * don't accidentally make it post error messages that will stop
+     * everything. */
+    if ((gst_element_set_state (element,
+                GST_STATE_READY)) == GST_STATE_CHANGE_FAILURE) {
+      GST_WARNING_OBJECT (dbin, "Couldn't set %s to READY",
+          GST_ELEMENT_NAME (element));
+      gst_object_unref (element);
+      continue;
+    }
+
+    /* 2.3. Find its sink pad, this should work after activating it. */
     if (!(sinkpad = find_sink_pad (element))) {
       GST_WARNING_OBJECT (dbin, "Element %s doesn't have a sink pad",
           GST_ELEMENT_NAME (element));
@@ -1041,7 +1102,7 @@ connect_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
     }
 
     /* 2.4 add it ... */
-    if (!(gst_bin_add (GST_BIN (dbin), element))) {
+    if (!(gst_bin_add (GST_BIN_CAST (dbin), element))) {
       GST_WARNING_OBJECT (dbin, "Couldn't add %s to the bin",
           GST_ELEMENT_NAME (element));
       gst_object_unref (sinkpad);
@@ -1049,16 +1110,6 @@ connect_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
       continue;
     }
 
-    /* ... activate it ... */
-    if ((gst_element_set_state (element,
-                GST_STATE_READY)) == GST_STATE_CHANGE_FAILURE) {
-      GST_WARNING_OBJECT (dbin, "Couldn't set %s to READY",
-          GST_ELEMENT_NAME (element));
-      gst_object_unref (sinkpad);
-      gst_bin_remove (GST_BIN (dbin), element);
-      continue;
-    }
-
     /* 2.5 ...and try to link */
     if ((gst_pad_link (pad, sinkpad)) != GST_PAD_LINK_OK) {
       GST_WARNING_OBJECT (dbin, "Link failed on pad %s:%s",
@@ -1088,10 +1139,10 @@ connect_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
     break;
   }
 
+beach:
   if (mqpad)
     gst_object_unref (mqpad);
 
-beach:
   return res;
 }
 
@@ -1407,49 +1458,13 @@ caps_notify_group_cb (GstPad * pad, GParamSpec * unused, GstDecodeGroup * group)
 static GValueArray *
 find_compatibles (GstDecodeBin * decode_bin, GstPad * pad, const GstCaps * caps)
 {
-  GList *factories;
-  GValueArray *to_try = g_value_array_new (0);
+  GValueArray *result;
 
-  /* loop over all the factories */
-  for (factories = decode_bin->factories; factories;
-      factories = g_list_next (factories)) {
-    GstElementFactory *factory = GST_ELEMENT_FACTORY (factories->data);
-    const GList *templates;
-    GList *walk;
+  GST_DEBUG_OBJECT (decode_bin, "finding factories");
 
-    /* get the templates from the element factory */
-    templates = gst_element_factory_get_static_pad_templates (factory);
-    for (walk = (GList *) templates; walk; walk = g_list_next (walk)) {
-      GstStaticPadTemplate *templ = walk->data;
+  result = gst_factory_list_filter (decode_bin->factories, caps);
 
-      /* we only care about the sink templates */
-      if (templ->direction == GST_PAD_SINK) {
-        GstCaps *intersect;
-        GstCaps *tmpl_caps;
-
-        /* try to intersect the caps with the caps of the template */
-        tmpl_caps = gst_static_caps_get (&templ->static_caps);
-
-        intersect = gst_caps_intersect (caps, tmpl_caps);
-        gst_caps_unref (tmpl_caps);
-
-        /* check if the intersection is empty */
-        if (!gst_caps_is_empty (intersect)) {
-          /* non empty intersection, we can use this element */
-          GValue val = { 0, };
-          g_value_init (&val, G_TYPE_OBJECT);
-          g_value_set_object (&val, factory);
-          g_value_array_append (to_try, &val);
-          g_value_unset (&val);
-          gst_caps_unref (intersect);
-          break;
-        }
-        gst_caps_unref (intersect);
-      }
-    }
-  }
-
-  return to_try;
+  return result;
 }
 
 /* Decide whether an element is a demuxer based on the 
@@ -1808,20 +1823,29 @@ gst_decode_group_check_if_drained (GstDecodeGroup * group)
   }
 
   group->drained = drained;
-  GST_LOG ("group is drained");
-
   if (!drained)
     return;
 
+  /* we are drained. Check if there is a next group to activate */
   DECODE_BIN_LOCK (dbin);
   if ((group == dbin->activegroup) && dbin->groups) {
     GST_DEBUG_OBJECT (dbin, "Switching to new group");
 
+    /* hide current group */
     gst_decode_group_hide (group);
-
+    /* expose next group */
     gst_decode_group_expose ((GstDecodeGroup *) dbin->groups->data);
+    /* we're not yet drained now */
+    drained = FALSE;
   }
   DECODE_BIN_UNLOCK (dbin);
+
+  if (drained) {
+    /* no more groups to activate, we're completely drained now */
+    GST_LOG ("all groups drained, fire signal");
+    g_signal_emit (G_OBJECT (dbin), gst_decode_bin_signals[SIGNAL_DRAINED], 0,
+        NULL);
+  }
 }
 
 /* sort_end_pads:
@@ -2189,10 +2213,14 @@ source_pad_event_probe (GstPad * pad, GstEvent * event, GstDecodePad * dpad)
     /* Set our pad as drained */
     dpad->drained = TRUE;
 
-    /* Check if all pads are drained */
+    GST_DEBUG_OBJECT (pad, "we received EOS");
+
+    /* Check if all pads are drained. If there is a next group to expose, we
+     * will remove the ghostpad of the current group first, which unlinks the
+     * peer and so drops the EOS. */
     gst_decode_group_check_if_drained (dpad->group);
   }
-
+  /* never drop events */
   return TRUE;
 }
 
@@ -2201,7 +2229,6 @@ source_pad_event_probe (GstPad * pad, GstEvent * event, GstDecodePad * dpad)
  * Creates a new GstDecodePad for the given pad.
  * If block is TRUE, Sets the pad blocking asynchronously
  */
-
 static GstDecodePad *
 gst_decode_pad_new (GstDecodeGroup * group, GstPad * pad, gboolean block)
 {
@@ -2292,9 +2319,6 @@ remove_fakesink (GstDecodeBin * decode_bin)
   gst_element_set_state (decode_bin->fakesink, GST_STATE_NULL);
   gst_bin_remove (GST_BIN (decode_bin), decode_bin->fakesink);
   decode_bin->fakesink = NULL;
-
-  gst_element_post_message (GST_ELEMENT_CAST (decode_bin),
-      gst_message_new_state_dirty (GST_OBJECT_CAST (decode_bin)));
 }
 
 /*****
@@ -2368,8 +2392,8 @@ missing_fakesink:
   }
 }
 
-static gboolean
-plugin_init (GstPlugin * plugin)
+gboolean
+gst_decode_bin_plugin_init (GstPlugin * plugin)
 {
   GST_DEBUG_CATEGORY_INIT (gst_decode_bin_debug, "decodebin2", 0,
       "decoder bin");
@@ -2383,9 +2407,3 @@ plugin_init (GstPlugin * plugin)
   return gst_element_register (plugin, "decodebin2", GST_RANK_NONE,
       GST_TYPE_DECODE_BIN);
 }
-
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
-    GST_VERSION_MINOR,
-    "decodebin2",
-    "decoder bin newer version", plugin_init, VERSION, GST_LICENSE,
-    GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
