@@ -26,7 +26,7 @@
  * audio and/or video player.
  * </para>
  * <para>
- * It can handle both audio and video files and features 
+ * It can handle both audio and video files and features
  * <itemizedlist>
  * <listitem>
  * automatic file type recognition and based on that automatic
@@ -161,28 +161,33 @@
  * new_state=PAUSED), since before that the list might not be complete yet or
  * not contain all available information (like language-codes).
  * </para>
- * <title>Buffering status</title>
+ * <title>Buffering</title>
  * <para>
- * Playbin handles buffering automatically. When playbin is buffering, it
- * will post BUFFERING messages on the bus with a percentage value that
- * shows the progress of the buffering. Applications may want to show this
- * information to the user in some form or another. Here is how to extract
- * the percentage information from the message:
+ * Playbin handles buffering automatically for the most part, but applications
+ * need to handle parts of the buffering process as well. Whenever playbin is
+ * buffering, it will post BUFFERING messages on the bus with a percentage
+ * value that shows the progress of the buffering process. Applications need
+ * to set playbin to PLAYING or PAUSED state in response to these messages.
+ * They may also want to convey the buffering progress to the user in some
+ * way. Here is how to extract the percentage information from the message
+ * (requires GStreamer >= 0.10.11):
  * </para>
  * <para>
  * <programlisting>
  * switch (GST_MESSAGE_TYPE (msg)) {
  *   case GST_MESSAGE_BUFFERING: {
  *     gint percent = 0;
- *     gst_structure_get_int (msg-&gt;structure, "buffer-percent", &amp;percent);
+ *     gst_message_parse_buffering (msg, &amp;percent);
  *     g_print ("Buffering (%%u percent done)", percent);
  *     break;
  *   }
  *   ...
  * }
  * </programlisting>
- * Note that application should keep the pipeline in the PAUSED state when a
- * GST_MESSAGE_BUFFERING message is received with a buffer-percent < 100.
+ * Note that applications should keep/set the pipeline in the PAUSED state when
+ * a BUFFERING message is received with a buffer percent value < 100 and set
+ * the pipeline back to PLAYING state when a BUFFERING message with a value
+ * of 100 percent is received (if PLAYING is the desired state, that is).
  * </para>
  * <title>Embedding the video window in your application</title>
  * <para>
@@ -241,6 +246,7 @@
 #include <gst/gst.h>
 
 #include <gst/gst-i18n-plugin.h>
+#include <gst/pbutils/pbutils.h>
 
 #include "gstplaybasebin.h"
 
@@ -254,7 +260,6 @@ GST_DEBUG_CATEGORY_STATIC (gst_play_bin_debug);
 #define GST_IS_PLAY_BIN_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_PLAY_BIN))
 
 #define VOLUME_MAX_DOUBLE 10.0
-#define CONNECTION_SPEED_DEFAULT 0
 
 typedef struct _GstPlayBin GstPlayBin;
 typedef struct _GstPlayBinClass GstPlayBinClass;
@@ -285,9 +290,6 @@ struct _GstPlayBin
   /* font description */
   gchar *font_desc;
 
-  /* connection speed in bits/sec (0 = unknown) */
-  guint connection_speed;
-
   /* indication if the pipeline is live */
   gboolean is_live;
 };
@@ -306,8 +308,7 @@ enum
   ARG_VIS_PLUGIN,
   ARG_VOLUME,
   ARG_FRAME,
-  ARG_FONT_DESC,
-  ARG_CONNECTION_SPEED
+  ARG_FONT_DESC
 };
 
 /* signals */
@@ -323,6 +324,8 @@ static void gst_play_bin_dispose (GObject * object);
 static gboolean setup_sinks (GstPlayBaseBin * play_base_bin,
     GstPlayBaseGroup * group);
 static void remove_sinks (GstPlayBin * play_bin);
+static void playbin_set_subtitles_visible (GstPlayBaseBin * play_base_bin,
+    gboolean visible);
 
 static void gst_play_bin_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * spec);
@@ -331,8 +334,6 @@ static void gst_play_bin_get_property (GObject * object, guint prop_id,
 
 static gboolean gst_play_bin_send_event (GstElement * element,
     GstEvent * event);
-static gboolean gst_play_bin_set_clock_func (GstElement * element,
-    GstClock * clock);
 static GstStateChangeReturn gst_play_bin_change_state (GstElement * element,
     GstStateChange transition);
 
@@ -346,7 +347,7 @@ static const GstElementDetails gst_play_bin_details =
 GST_ELEMENT_DETAILS ("Player Bin",
     "Generic/Bin/Player",
     "Autoplug and play media from an uri",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 static GType
 gst_play_bin_get_type (void)
@@ -416,17 +417,6 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
           "Subtitle font description",
           "Pango font description of font "
           "to be used for subtitle rendering", NULL, G_PARAM_WRITABLE));
-  /**
-   * GstPlayBin::connection-speed
-   *
-   * Network connection speed in kbps (0 = unknown)
-   *
-   * Since: 0.10.10
-   **/
-  g_object_class_install_property (gobject_klass, ARG_CONNECTION_SPEED,
-      g_param_spec_uint ("connection-speed", "Connection Speed",
-          "Network connection speed in kbps (0 = unknown)",
-          0, G_MAXUINT, CONNECTION_SPEED_DEFAULT, G_PARAM_READWRITE));
 
   gobject_klass->dispose = GST_DEBUG_FUNCPTR (gst_play_bin_dispose);
 
@@ -435,12 +425,12 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
   gstelement_klass->change_state =
       GST_DEBUG_FUNCPTR (gst_play_bin_change_state);
   gstelement_klass->send_event = GST_DEBUG_FUNCPTR (gst_play_bin_send_event);
-  gstelement_klass->set_clock = GST_DEBUG_FUNCPTR (gst_play_bin_set_clock_func);
 
   gstbin_klass->handle_message =
       GST_DEBUG_FUNCPTR (gst_play_bin_handle_message);
 
   playbasebin_klass->setup_output_pads = setup_sinks;
+  playbasebin_klass->set_subtitles_visible = playbin_set_subtitles_visible;
 }
 
 static void
@@ -507,7 +497,11 @@ static void
 gst_play_bin_vis_unblocked (GstPad * tee_pad, gboolean blocked,
     gpointer user_data)
 {
-  /* Unblocked */
+  GstPlayBin *play_bin = GST_PLAY_BIN (user_data);
+
+  if (play_bin->pending_visualisation)
+    gst_pad_set_blocked_async (tee_pad, FALSE, gst_play_bin_vis_unblocked,
+        play_bin);
 }
 
 static void
@@ -518,9 +512,15 @@ gst_play_bin_vis_blocked (GstPad * tee_pad, gboolean blocked,
   GstBin *vis_bin = NULL;
   GstPad *vis_sink_pad = NULL, *vis_src_pad = NULL, *vqueue_pad = NULL;
   GstState bin_state;
+  GstElement *pending_visualisation;
+
+  GST_OBJECT_LOCK (play_bin);
+  pending_visualisation = play_bin->pending_visualisation;
+  play_bin->pending_visualisation = NULL;
+  GST_OBJECT_UNLOCK (play_bin);
 
   /* We want to disable visualisation */
-  if (!GST_IS_ELEMENT (play_bin->pending_visualisation)) {
+  if (!GST_IS_ELEMENT (pending_visualisation)) {
     /* Set visualisation element to READY */
     gst_element_set_state (play_bin->visualisation, GST_STATE_READY);
     goto beach;
@@ -569,17 +569,16 @@ gst_play_bin_vis_blocked (GstPad * tee_pad, gboolean blocked,
   /* And loose our ref */
   gst_object_unref (play_bin->visualisation);
 
-  if (play_bin->pending_visualisation) {
+  if (pending_visualisation) {
     /* Ref this new visualisation element before adding to the bin */
-    gst_object_ref (play_bin->pending_visualisation);
+    gst_object_ref (pending_visualisation);
     /* Add the new one */
-    gst_bin_add (vis_bin, play_bin->pending_visualisation);
+    gst_bin_add (vis_bin, pending_visualisation);
     /* Synchronizing state */
-    gst_element_set_state (play_bin->pending_visualisation, bin_state);
+    gst_element_set_state (pending_visualisation, bin_state);
 
-    vis_sink_pad = gst_element_get_pad (play_bin->pending_visualisation,
-        "sink");
-    vis_src_pad = gst_element_get_pad (play_bin->pending_visualisation, "src");
+    vis_sink_pad = gst_element_get_pad (pending_visualisation, "sink");
+    vis_src_pad = gst_element_get_pad (pending_visualisation, "src");
 
     if (!GST_IS_PAD (vis_sink_pad) || !GST_IS_PAD (vis_src_pad)) {
       goto beach;
@@ -592,8 +591,7 @@ gst_play_bin_vis_blocked (GstPad * tee_pad, gboolean blocked,
 
   /* We are done */
   gst_object_unref (play_bin->visualisation);
-  play_bin->visualisation = play_bin->pending_visualisation;
-  play_bin->pending_visualisation = NULL;
+  play_bin->visualisation = pending_visualisation;
 
 beach:
   if (vis_sink_pad) {
@@ -633,7 +631,7 @@ gst_play_bin_set_property (GObject * object, guint prop_id,
         gst_object_sink (GST_OBJECT_CAST (play_bin->video_sink));
       }
       /* when changing the videosink, we just remove the
-       * video pipeline from the cache so that it will be 
+       * video pipeline from the cache so that it will be
        * regenerated with the new sink element */
       g_hash_table_remove (play_bin->cache, "vbin");
       break;
@@ -650,24 +648,23 @@ gst_play_bin_set_property (GObject * object, guint prop_id,
       break;
     case ARG_VIS_PLUGIN:
     {
+      GstElement *pending_visualisation =
+          GST_ELEMENT_CAST (g_value_get_object (value));
+
+      /* Take ownership */
+      if (pending_visualisation) {
+        gst_object_ref (pending_visualisation);
+        gst_object_sink (pending_visualisation);
+      }
+
       /* Do we already have a visualisation change pending ? */
+      GST_OBJECT_LOCK (play_bin);
       if (play_bin->pending_visualisation) {
         gst_object_unref (play_bin->pending_visualisation);
-        play_bin->pending_visualisation = g_value_get_object (value);
-        /* Take ownership */
-        if (play_bin->pending_visualisation) {
-          gst_object_ref (play_bin->pending_visualisation);
-          gst_object_sink (GST_OBJECT_CAST (play_bin->pending_visualisation));
-        }
+        play_bin->pending_visualisation = pending_visualisation;
+        GST_OBJECT_UNLOCK (play_bin);
       } else {
-        play_bin->pending_visualisation = g_value_get_object (value);
-
-        /* Take ownership */
-        if (play_bin->pending_visualisation) {
-          gst_object_ref (play_bin->pending_visualisation);
-          gst_object_sink (GST_OBJECT_CAST (play_bin->pending_visualisation));
-        }
-
+        GST_OBJECT_UNLOCK (play_bin);
         /* Was there a visualisation already set ? */
         if (play_bin->visualisation != NULL) {
           GstBin *vis_bin = NULL;
@@ -691,6 +688,7 @@ gst_play_bin_set_property (GObject * object, guint prop_id,
               goto beach;
             }
 
+            play_bin->pending_visualisation = pending_visualisation;
             /* Block with callback */
             gst_pad_set_blocked_async (tee_pad, TRUE, gst_play_bin_vis_blocked,
                 play_bin);
@@ -703,12 +701,10 @@ gst_play_bin_set_property (GObject * object, guint prop_id,
             }
             gst_object_unref (vis_bin);
           } else {
-            play_bin->visualisation = play_bin->pending_visualisation;
-            play_bin->pending_visualisation = NULL;
+            play_bin->visualisation = pending_visualisation;
           }
         } else {
-          play_bin->visualisation = play_bin->pending_visualisation;
-          play_bin->pending_visualisation = NULL;
+          play_bin->visualisation = pending_visualisation;
         }
       }
       break;
@@ -728,9 +724,6 @@ gst_play_bin_set_property (GObject * object, guint prop_id,
             "font-desc", g_value_get_string (value), NULL);
       }
       break;
-    case ARG_CONNECTION_SPEED:
-      play_bin->connection_speed = g_value_get_uint (value) * 1000;
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -758,12 +751,13 @@ gst_play_bin_get_property (GObject * object, guint prop_id, GValue * value,
     case ARG_VOLUME:
       g_value_set_double (value, play_bin->volume);
       break;
-    case ARG_FRAME:
-      gst_value_set_mini_object (value, GST_MINI_OBJECT (play_bin->frame));
-      break;
-    case ARG_CONNECTION_SPEED:
-      g_value_set_uint (value, play_bin->connection_speed / 1000);
+    case ARG_FRAME:{
+      GstBuffer *cur_frame = NULL;
+
+      gst_buffer_replace (&cur_frame, play_bin->frame);
+      gst_value_take_buffer (value, cur_frame);
       break;
+    }
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -777,21 +771,28 @@ static void
 handoff (GstElement * identity, GstBuffer * frame, gpointer data)
 {
   GstPlayBin *play_bin = GST_PLAY_BIN (data);
-  GstBuffer **frame_p = &play_bin->frame;
-
-  gst_mini_object_replace ((GstMiniObject **) frame_p,
-      GST_MINI_OBJECT_CAST (frame));
 
   /* applications need to know the buffer caps,
    * make sure they are always set on the frame */
-  if (GST_BUFFER_CAPS (play_bin->frame) == NULL) {
+  if (GST_BUFFER_CAPS (frame) == NULL) {
     GstPad *pad;
 
     if ((pad = gst_element_get_pad (identity, "sink"))) {
-      gst_buffer_set_caps (play_bin->frame, GST_PAD_CAPS (pad));
+      gst_buffer_set_caps (frame, GST_PAD_CAPS (pad));
       gst_object_unref (pad);
     }
   }
+
+  gst_buffer_replace (&play_bin->frame, frame);
+}
+
+static void
+post_missing_element_message (GstPlayBin * playbin, const gchar * name)
+{
+  GstMessage *msg;
+
+  msg = gst_missing_element_message_new (GST_ELEMENT_CAST (playbin), name);
+  gst_element_post_message (GST_ELEMENT_CAST (playbin), msg);
 }
 
 /* make the element (bin) that contains the elements needed to perform
@@ -881,8 +882,7 @@ gen_video_element (GstPlayBin * play_bin)
   /* ERRORS */
 no_sinks:
   {
-    /* FIXME: this warrants adding a CORE error category for missing
-     * elements/plugins */
+    post_missing_element_message (play_bin, "autovideosink");
     GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
         (_("Both autovideosink and xvimagesink elements are missing.")),
         (NULL));
@@ -890,6 +890,7 @@ no_sinks:
   }
 no_colorspace:
   {
+    post_missing_element_message (play_bin, "ffmpegcolorspace");
     GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
         (_("Missing element '%s' - check your GStreamer installation."),
             "ffmpegcolorspace"), (NULL));
@@ -899,6 +900,7 @@ no_colorspace:
 
 no_videoscale:
   {
+    post_missing_element_message (play_bin, "videoscale");
     GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
         (_("Missing element '%s' - check your GStreamer installation."),
             "videoscale"), ("possibly a liboil version mismatch?"));
@@ -986,6 +988,7 @@ gen_text_element (GstPlayBin * play_bin)
   /* ERRORS */
 no_overlay:
   {
+    post_missing_element_message (play_bin, "textoverlay");
     GST_WARNING_OBJECT (play_bin,
         "No overlay (pango) element, subtitles disabled");
     return vbin;
@@ -1075,12 +1078,14 @@ gen_audio_element (GstPlayBin * play_bin)
   /* ERRORS */
 no_sinks:
   {
+    post_missing_element_message (play_bin, "autoaudiosink");
     GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
         (_("Both autoaudiosink and alsasink elements are missing.")), (NULL));
     return NULL;
   }
 no_audioconvert:
   {
+    post_missing_element_message (play_bin, "audioconvert");
     GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
         (_("Missing element '%s' - check your GStreamer installation."),
             "audioconvert"), ("possibly a liboil version mismatch?"));
@@ -1090,6 +1095,7 @@ no_audioconvert:
 
 no_audioresample:
   {
+    post_missing_element_message (play_bin, "audioresample");
     GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
         (_("Missing element '%s' - check your GStreamer installation."),
             "audioresample"), ("possibly a liboil version mismatch?"));
@@ -1106,27 +1112,27 @@ link_failed:
 }
 
 /* make the element (bin) that contains the elements needed to perform
- * visualisation ouput.  The idea is to split the audio using tee, then 
+ * visualisation ouput.  The idea is to split the audio using tee, then
  * sending the output to the regular audio bin and the other output to
  * the vis plugin that transforms it into a video that is rendered with the
- * normal video bin. The video bin is run in a thread to make sure it does
- * not block the audio playback pipeline.
+ * normal video bin. The video and audio bins are run in threads to make sure
+ * they don't block eachother.
  *
- *  +--------------------------------------------------------------------+
- *  | visbin                                                             |
- *  |      +------+   +--------+   +----------------+                    |
- *  |      | tee  |   | aqueue |   |   abin ...     |                    |
- *  |   +-sink   src-sink     src-sink              |                    |
- *  |   |  |      |   +--------+   +----------------+                    |
- *  |   |  |      |                                                      |
- *  |   |  |      |   +------+   +---------+   +------+   +-----------+  |
- *  |   |  |      |   |vqueue|   |audioconv|   | vis  |   | vbin ...  |  |
- *  |   |  |     src-sink   src-sink      src-sink   src-sink         |  |
- *  |   |  |      |   +------+   +---------+   +------+   +-----------+  |
- *  |   |  |      |                                                      |
- *  |   |  +------+                                                      |
- * sink-+                                                                |
-   +---------------------------------------------------------------------+
+ *  +-----------------------------------------------------------------------+
+ *  | visbin                                                                |
+ *  |      +------+   +--------+   +----------------+                       |
+ *  |      | tee  |   | aqueue |   |   abin ...     |                       |
+ *  |   +-sink   src-sink     src-sink              |                       |
+ *  |   |  |      |   +--------+   +----------------+                       |
+ *  |   |  |      |                                                         |
+ *  |   |  |      |   +------+   +------------+   +------+   +-----------+  |
+ *  |   |  |      |   |vqueue|   | audioconv  |   | vis  |   | vbin ...  |  |
+ *  |   |  |     src-sink   src-sink + samp  src-sink   src-sink         |  |
+ *  |   |  |      |   +------+   +------------+   +------+   +-----------+  |
+ *  |   |  |      |                                                         |
+ *  |   |  +------+                                                         |
+ * sink-+                                                                   |
+ *  +-----------------------------------------------------------------------+
  */
 static GstElement *
 gen_vis_element (GstPlayBin * play_bin)
@@ -1137,6 +1143,8 @@ gen_vis_element (GstPlayBin * play_bin)
   GstElement *asink;
   GstElement *vsink;
   GstElement *conv;
+  GstElement *resamp;
+  GstElement *conv2;
   GstElement *vis;
   GstElement *vqueue, *aqueue;
   GstPad *pad, *rpad;
@@ -1168,6 +1176,16 @@ gen_vis_element (GstPlayBin * play_bin)
     goto no_audioconvert;
   gst_bin_add (GST_BIN_CAST (element), conv);
 
+  resamp = gst_element_factory_make ("audioresample", "aresamp");
+  if (resamp == NULL)
+    goto no_audioresample;
+  gst_bin_add (GST_BIN_CAST (element), resamp);
+
+  conv2 = gst_element_factory_make ("audioconvert", "aconv2");
+  if (conv2 == NULL)
+    goto no_audioconvert;
+  gst_bin_add (GST_BIN_CAST (element), conv2);
+
   if (play_bin->visualisation) {
     gst_object_ref (play_bin->visualisation);
     vis = play_bin->visualisation;
@@ -1179,7 +1197,9 @@ gen_vis_element (GstPlayBin * play_bin)
   gst_bin_add (GST_BIN_CAST (element), vis);
 
   res = gst_element_link_pads (vqueue, "src", conv, "sink");
-  res &= gst_element_link_pads (conv, "src", vis, "sink");
+  res &= gst_element_link_pads (conv, "src", resamp, "sink");
+  res &= gst_element_link_pads (resamp, "src", conv2, "sink");
+  res &= gst_element_link_pads (conv2, "src", vis, "sink");
   res &= gst_element_link_pads (vis, "src", vsink, "sink");
   if (!res)
     goto link_failed;
@@ -1206,14 +1226,25 @@ gen_vis_element (GstPlayBin * play_bin)
   /* ERRORS */
 no_audioconvert:
   {
+    post_missing_element_message (play_bin, "audioconvert");
     GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
         (_("Missing element '%s' - check your GStreamer installation."),
             "audioconvert"), ("possibly a liboil version mismatch?"));
     gst_object_unref (element);
     return NULL;
   }
+no_audioresample:
+  {
+    post_missing_element_message (play_bin, "audioresample");
+    GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
+        (_("Missing element '%s' - check your GStreamer installation."),
+            "audioresample"), (NULL));
+    gst_object_unref (element);
+    return NULL;
+  }
 no_goom:
   {
+    post_missing_element_message (play_bin, "goom");
     GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
         (_("Missing element '%s' - check your GStreamer installation."),
             "goom"), (NULL));
@@ -1335,7 +1366,7 @@ remove_sinks (GstPlayBin * play_bin)
  * media file. First we count the number of audio and video streams.
  * If there is no video stream but there exists an audio stream,
  * we install a visualisation pipeline.
- * 
+ *
  * Also make sure to only connect the first audio and video pad. FIXME
  * this should eventually be handled with a tuner interface so that
  * one can switch the streams.
@@ -1354,28 +1385,19 @@ add_sink (GstPlayBin * play_bin, GstElement * sink, GstPad * srcpad,
 
   g_return_val_if_fail (sink != NULL, FALSE);
 
-  /* For live pipelines we need to add the bin in the same state as the 
-   * parent so that it starts as soon as it is prerolled. */
-  if (play_bin->is_live)
-    state = GST_STATE_PLAYING;
-  else
-    state = GST_STATE_PAUSED;
+  state = GST_STATE_PAUSED;
 
   /* this is only for debugging */
   parent = gst_pad_get_parent_element (srcpad);
   if (parent) {
-    GST_DEBUG ("Adding sink with state %d (parent: %d, peer: %d)",
+    GST_DEBUG ("Adding sink %" GST_PTR_FORMAT
+        " with state %d (parent: %d, peer: %d)", sink,
         GST_STATE (sink), GST_STATE (play_bin), GST_STATE (parent));
     gst_object_unref (parent);
   }
   gst_bin_add (GST_BIN_CAST (play_bin), sink);
 
-  /* for live pipelines, disable the sync in the sinks until core handles this
-   * correctly. */
-  if (play_bin->is_live)
-    gst_element_set_clock (sink, NULL);
-
-  /* bring it to the PAUSED state so we can link to the peer without
+  /* bring it to the required state so we can link to the peer without
    * breaking the flow */
   stateret = gst_element_set_state (sink, state);
   if (stateret == GST_STATE_CHANGE_FAILURE)
@@ -1436,16 +1458,12 @@ link_failed:
   }
 subtitle_failed:
   {
-    gchar *capsstr;
     GstCaps *caps;
 
     /* could not link this stream */
     caps = gst_pad_get_caps (subtitle_pad);
-    capsstr = gst_caps_to_string (caps);
-    GST_DEBUG_OBJECT (play_bin,
-        "subtitle link failed when adding sink, caps %s, reason %d", capsstr,
-        linkres);
-    g_free (capsstr);
+    GST_WARNING_OBJECT (play_bin, "subtitle link failed when adding sink, "
+        "caps = %" GST_PTR_FORMAT ", reason %d", caps, linkres);
     gst_caps_unref (caps);
 
     /* not fatal */
@@ -1453,6 +1471,11 @@ subtitle_failed:
   }
 }
 
+static void
+dummy_blocked_cb (GstPad * pad, gboolean blocked, gpointer user_data)
+{
+}
+
 static gboolean
 setup_sinks (GstPlayBaseBin * play_base_bin, GstPlayBaseGroup * group)
 {
@@ -1460,7 +1483,7 @@ setup_sinks (GstPlayBaseBin * play_base_bin, GstPlayBaseGroup * group)
   GList *streaminfo = NULL, *s;
   gboolean need_vis = FALSE;
   gboolean need_text = FALSE;
-  GstPad *textsrcpad = NULL, *pad = NULL;
+  GstPad *textsrcpad = NULL, *pad = NULL, *origtextsrcpad = NULL;
   GstElement *sink;
   gboolean res = TRUE;
 
@@ -1517,6 +1540,7 @@ setup_sinks (GstPlayBaseBin * play_base_bin, GstPlayBaseGroup * group)
       textsrcpad =
           gst_element_get_pad (group->type[GST_STREAM_TYPE_TEXT - 1].preroll,
           "src");
+
       /* This pad is from subtitle-bin, we need to create a ghost pad to have
          common grandparents */
       parent = gst_object_get_parent (GST_OBJECT_CAST (textsrcpad));
@@ -1541,6 +1565,11 @@ setup_sinks (GstPlayBaseBin * play_base_bin, GstPlayBaseGroup * group)
       if (!GST_IS_PLAY_BIN (grandparent)) {
         GST_DEBUG_OBJECT (textsrcpad, "this subtitle pad is from a subtitle "
             "file, ghosting to a suitable hierarchy");
+        /* Block the pad first, because as soon as we add a ghostpad, the queue
+         * will try and start pushing */
+        gst_pad_set_blocked_async (textsrcpad, TRUE, dummy_blocked_cb, NULL);
+        origtextsrcpad = gst_object_ref (textsrcpad);
+
         ghost = gst_ghost_pad_new ("text_src", textsrcpad);
         if (!GST_IS_PAD (ghost)) {
           GST_WARNING_OBJECT (textsrcpad, "failed creating ghost pad for "
@@ -1582,6 +1611,10 @@ setup_sinks (GstPlayBaseBin * play_base_bin, GstPlayBaseGroup * group)
     gst_object_unref (pad);
     if (textsrcpad)
       gst_object_unref (textsrcpad);
+    if (origtextsrcpad) {
+      gst_pad_set_blocked_async (origtextsrcpad, FALSE, dummy_blocked_cb, NULL);
+      gst_object_unref (origtextsrcpad);
+    }
   }
 
   /* remove the sinks now, pipeline get_state will now wait for the
@@ -1595,6 +1628,20 @@ setup_sinks (GstPlayBaseBin * play_base_bin, GstPlayBaseGroup * group)
   return res;
 }
 
+static void
+playbin_set_subtitles_visible (GstPlayBaseBin * play_base_bin, gboolean visible)
+{
+  GstPlayBin *playbin = GST_PLAY_BIN (play_base_bin);
+
+  /* we're ignoring the case of someone setting the 'current-text' property
+   * before textoverlay is set up (which is probably okay, since playbasebin
+   * will just select the first subtitle stream as active stream regardless) */
+  if (playbin->textoverlay_element != NULL) {
+    GST_LOG_OBJECT (playbin, "setting subtitle visibility to %d", visible);
+    g_object_set (playbin->textoverlay_element, "silent", !visible, NULL);
+  }
+}
+
 /* Send an event to our sinks until one of them works; don't then send to the
  * remaining sinks (unlike GstBin)
  */
@@ -1624,49 +1671,7 @@ gst_play_bin_send_event_to_sink (GstPlayBin * play_bin, GstEvent * event)
   return res;
 }
 
-static gboolean
-do_playbin_seek (GstElement * element, GstEvent * event)
-{
-  gdouble rate;
-  GstSeekFlags flags;
-  gboolean flush;
-  gboolean was_playing = FALSE;
-  gboolean res;
-
-  gst_event_parse_seek (event, &rate, NULL, &flags, NULL, NULL, NULL, NULL);
-
-  flush = flags & GST_SEEK_FLAG_FLUSH;
-
-  if (flush) {
-    GstState state;
-
-    /* need to call _get_state() since a bin state is only updated
-     * with this call. */
-    gst_element_get_state (element, &state, NULL, 0);
-    was_playing = state == GST_STATE_PLAYING;
-
-    if (was_playing) {
-      gst_element_set_state (element, GST_STATE_PAUSED);
-      gst_element_get_state (element, NULL, NULL, 50 * GST_MSECOND);
-    }
-  }
-
-  GST_DEBUG_OBJECT (element, "Sending seek event to a sink");
-  res = gst_play_bin_send_event_to_sink (GST_PLAY_BIN (element), event);
-
-  if (flush) {
-    /* need to reset the stream time to 0 after a flushing seek */
-    if (res)
-      gst_pipeline_set_new_stream_time (GST_PIPELINE (element), 0);
-
-    if (was_playing)
-      /* and continue playing */
-      gst_element_set_state (element, GST_STATE_PLAYING);
-  }
-  return res;
-}
-
-/* We only want to send the event to a single sink (overriding GstBin's 
+/* We only want to send the event to a single sink (overriding GstBin's
  * behaviour), but we want to keep GstPipeline's behaviour - wrapping seek
  * events appropriately. So, this is a messy duplication of code. */
 static gboolean
@@ -1675,55 +1680,19 @@ gst_play_bin_send_event (GstElement * element, GstEvent * event)
   gboolean res = FALSE;
   GstEventType event_type = GST_EVENT_TYPE (event);
 
-
   switch (event_type) {
     case GST_EVENT_SEEK:
-      res = do_playbin_seek (element, event);
-      break;
-    default:
+      GST_DEBUG_OBJECT (element, "Sending seek event to a sink");
       res = gst_play_bin_send_event_to_sink (GST_PLAY_BIN (element), event);
       break;
+    default:
+      res = parent_class->send_event (element, event);
+      break;
   }
 
   return res;
 }
 
-/* Override the set_clock function, we don't want to set a clock on the sinks
- * when we are live pipeline so that they don't synchronize until this is
- * fixed in core. */
-static gboolean
-gst_play_bin_set_clock_func (GstElement * element, GstClock * clock)
-{
-  GList *children;
-  GstBin *bin;
-  GstPlayBin *play_bin;
-  gboolean res = TRUE;
-  GstElement *asink, *vsink;
-
-  bin = GST_BIN (element);
-  play_bin = GST_PLAY_BIN (element);
-
-  asink = g_hash_table_lookup (play_bin->cache, "audio_sink");
-  vsink = g_hash_table_lookup (play_bin->cache, "video_sink");
-
-  GST_DEBUG_OBJECT (play_bin, "setting clock, is_live %d", play_bin->is_live);
-
-  GST_OBJECT_LOCK (bin);
-  if (element->clock != clock) {
-    for (children = bin->children; children; children = g_list_next (children)) {
-      GstElement *child = GST_ELEMENT (children->data);
-
-      if (play_bin->is_live && (child == asink || child == vsink))
-        res &= gst_element_set_clock (child, NULL);
-      else
-        res &= gst_element_set_clock (child, clock);
-    }
-  }
-  GST_OBJECT_UNLOCK (bin);
-
-  return res;
-}
-
 static void
 value_list_append_structure_list (GValue * list_val, GstStructure ** first,
     GList * structure_list)
@@ -1755,11 +1724,13 @@ gst_play_bin_handle_redirect_message (GstPlayBin * playbin, GstMessage * msg)
   GList *l_good = NULL, *l_neutral = NULL, *l_bad = NULL;
   GValue new_list = { 0, };
   guint size, i;
+  GstPlayBaseBin *playbasebin = GST_PLAY_BASE_BIN (playbin);
+  guint connection_speed = playbasebin->connection_speed;
 
   GST_DEBUG_OBJECT (playbin, "redirect message: %" GST_PTR_FORMAT, msg);
-  GST_DEBUG_OBJECT (playbin, "connection speed: %u", playbin->connection_speed);
+  GST_DEBUG_OBJECT (playbin, "connection speed: %u", connection_speed);
 
-  if (playbin->connection_speed == 0 || msg->structure == NULL)
+  if (connection_speed == 0 || msg->structure == NULL)
     return msg;
 
   locations_list = gst_structure_get_value (msg->structure, "locations");
@@ -1784,10 +1755,10 @@ gst_play_bin_handle_redirect_message (GstPlayBin * playbin, GstMessage * msg)
     if (!gst_structure_get_int (s, "minimum-bitrate", &bitrate) || bitrate <= 0) {
       GST_DEBUG_OBJECT (playbin, "no bitrate: %" GST_PTR_FORMAT, s);
       l_neutral = g_list_append (l_neutral, (gpointer) s);
-    } else if (bitrate > playbin->connection_speed) {
+    } else if (bitrate > connection_speed) {
       GST_DEBUG_OBJECT (playbin, "bitrate too high: %" GST_PTR_FORMAT, s);
       l_bad = g_list_append (l_bad, (gpointer) s);
-    } else if (bitrate <= playbin->connection_speed) {
+    } else if (bitrate <= connection_speed) {
       GST_DEBUG_OBJECT (playbin, "bitrate OK: %" GST_PTR_FORMAT, s);
       l_good = g_list_append (l_good, (gpointer) s);
     }
@@ -1858,6 +1829,7 @@ gst_play_bin_change_state (GstElement * element, GstStateChange transition)
       /* FIXME Release audio device when we implement that */
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+    case GST_STATE_CHANGE_READY_TO_NULL:
       /* remove sinks we added */
       remove_sinks (play_bin);
       /* and there might be a fakesink we need to clean up now */
@@ -1874,23 +1846,11 @@ gst_play_bin_change_state (GstElement * element, GstStateChange transition)
   return ret;
 }
 
-static gboolean
-plugin_init (GstPlugin * plugin)
+gboolean
+gst_play_bin_plugin_init (GstPlugin * plugin)
 {
   GST_DEBUG_CATEGORY_INIT (gst_play_bin_debug, "playbin", 0, "play bin");
 
-#ifdef ENABLE_NLS
-  GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE,
-      LOCALEDIR);
-  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
-#endif /* ENABLE_NLS */
-
   return gst_element_register (plugin, "playbin", GST_RANK_NONE,
       GST_TYPE_PLAY_BIN);
 }
-
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
-    GST_VERSION_MINOR,
-    "playbin",
-    "player bin", plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME,
-    GST_PACKAGE_ORIGIN)
